// © 2023 John Breauxs
//! Common error type for [msp430-asm](crate) errors

use super::*;
use std::fmt::Display;

#[derive(Debug)]
pub enum Error {
    /// Produced by [lexer]
    LexError(lexer::error::LexError),
    /// Produced by [parser]
    ParseError(parser::error::ParseError),
    /// Produced by [assembler]
    AssemblyError(assembler::error::AssemblyError),
}

impl Error {}

impl From<lexer::error::LexError> for Error {
    fn from(value: lexer::error::LexError) -> Self { Self::LexError(value) }
}

impl From<parser::error::ParseError> for Error {
    fn from(value: parser::error::ParseError) -> Self { Self::ParseError(value) }
}

impl From<assembler::error::AssemblyError> for Error {
    fn from(value: assembler::error::AssemblyError) -> Self { Self::AssemblyError(value) }
}

impl Display for Error {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Error::LexError(e) => Display::fmt(e, f),
            Error::ParseError(e) => Display::fmt(e, f),
            Error::AssemblyError(e) => Display::fmt(e, f),
        }
    }
}

impl std::error::Error for Error {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Error::LexError(e) => Some(e),
            Error::ParseError(e) => Some(e),
            Error::AssemblyError(e) => Some(e),
        }
    }
}
